from kivy.factory import Factory

r = Factory.register
r("AKAddWidgetAnimationBehavior", module="akivymd.uix.behaviors.addwidget")
r("AKBadgeLayout", module="akivymd.uix.badgelayout")
r("AKFloatingRoundedAppbar", module="akivymd.uix.bottomappbar")
r("AKFloatingRoundedAppbarAvatarItem", module="akivymd.uix.bottomappbar")
r("AKFloatingRoundedAppbarButtonItem", module="akivymd.uix.bottomappbar")
r("AKBottomNavigation", module="akivymd.uix.bottomnavigation")
r("AKBottomNavigation2", module="akivymd.uix.bottomnavigation2")
r("Button_Item", module="akivymd.uix.bottomnavigation2")
r("AKImageLoader", module="akivymd.uix.imageloader")
r("AKLabelLoader", module="akivymd.uix.labelloader")
r("AKImageViewer", module="akivymd.uix.imageview")
r("AKImageViewerItem", module="akivymd.uix.imageview")
r("AKOnboarding", module="akivymd.uix.onboarding")
r("AKOnboardingItem", module="akivymd.uix.onboarding")
r("AKPieChart", module="akivymd.uix.piechart")
r("AKSelectList", module="akivymd.uix.selectionlist")
r("AKSelectListAvatarItem", module="akivymd.uix.selectionlist")
r("AKProgressbutton", module="akivymd.uix.progressbutton")
r("AKSilverAppbar", module="akivymd.uix.silverappbar")
r("AKSilverAppbarContent", module="akivymd.uix.silverappbar")
r("AKSilverAppbarHeader", module="akivymd.uix.silverappbar")
r("AKSpinnerDoubleBounce", module="akivymd.uix.spinners")
r("AKSpinnerCircleFlip", module="akivymd.uix.spinners")
r("AKSpinnerFoldingCube", module="akivymd.uix.spinners")
r("AKSpinnerThreeDots", module="akivymd.uix.spinners")



from kivy.lang.builder import Builder 
from kivy.uix.boxlayout import BoxLayout
from kivymd.theming import ThemableBehavior
from kivy.properties import NumericProperty, StringProperty, ListProperty, BooleanProperty
from kivymd.uix.toolbar import MDToolbar
from kivy.uix.scrollview import ScrollView
from kivy.clock import Clock
from kivy.animation import Animation

Builder.load_string(
    """
#:import ScrollEffect kivy.effects.scroll.ScrollEffect
<AKSilverAppbarContent>
    canvas.before:
        Color: 
            rgba: root.md_bg_color if root.md_bg_color else root.theme_cls.bg_normal
        RoundedRectangle:
            pos: self.pos 
            size : self.size 
            radius: [root.parent.parent.parent.parent.radius, root.parent.parent.parent.parent.radius, 0,0]

<AKSilverAppbar>

    FloatLayout:
        id: float_box    
        BoxLayout:
            canvas.after:
                Color: 
                    rgba: 0,0,0,0
                    a: root._darkness
                Rectangle:
                    pos: self.pos 
                    size: self.size         
            id: header
            size_hint_y: None 
            height: root.max_height +  root.radius 
            pos: self.x, root.height-root.max_height- root.radius   
                    
        NewScrollView:
            effect_cls:ScrollEffect

            MDBoxLayout:
                id: scroll_box
                adaptive_height: True 
                orientation: 'vertical'

                BoxLayout:
                    size_hint_y: None 
                    height: root.max_height

        MDToolbar:
            id: toolbar
            pos: self.x, float_box.height-self.height
            right_action_items: root.right_action_items
            left_action_items: root.left_action_items  
            title: root.title
            anchor_title: root.anchor_title
            md_bg_color: root.toolbar_bg if root.toolbar_bg else root.theme_cls.primary_color
            elevation: 0.01 if root.elevation==0 else root.elevation

    """
)

class AKSilverAppbarContent(ThemableBehavior, BoxLayout):
    md_bg_color= ListProperty() 

class AKSilverAppbarHeader(BoxLayout):
    pass 

class NewScrollView(ScrollView):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda x: self._update())

    def on_vbar(self,*args):
        toolbar_percent= (self.root.ids.toolbar.height/self.parent.parent.ids.scroll_box.height)*100
        current_percent=  (self.vbar[0]+self.vbar[1])*100
        banner_percent_min=  (1- self.parent.parent.max_height/self.parent.parent.ids.scroll_box.height)*100+toolbar_percent
        if self.root.hide_toolbar:
            if banner_percent_min<=current_percent:
                current_percent_in_banner= current_percent- banner_percent_min
                opacity= current_percent_in_banner/(100-banner_percent_min)

                self.root._darkness= self.root.header_max_darkness*(1-opacity)

                if not self.root.pin_top:
                    self.root.toolbar_bg= self.root.toolbar_bg[0:3]+[0]
                    self.root.ids.toolbar.opacity= opacity
                else:
                    self.root.toolbar_bg= self.root.toolbar_bg[0:3]+[1-opacity]
                    self.root.ids.toolbar._hard_shadow_a= 1-opacity
                    self.root.ids.toolbar._soft_shadow_a= 1-opacity

            else:
                if not self.root.pin_top:
                    self.root.ids.toolbar.opacity= 0
                else: 
                    self.root.toolbar_bg= self.root.toolbar_bg[0:3]+[1]

    def _update(self):
        self.root= self.parent.parent
        
class AKSilverAppbar(ThemableBehavior, BoxLayout):

    max_height= NumericProperty(300) 
    left_action_items= ListProperty()
    right_action_items= ListProperty()
    title = StringProperty()
    toolbar_bg = ListProperty()
    anchor_title = StringProperty("left")
    pin_top= BooleanProperty(True)
    hide_toolbar= BooleanProperty(True)
    elevation= NumericProperty(6)
    radius= NumericProperty(20)
    header_max_darkness= NumericProperty(0.7)

    _darkness= NumericProperty()
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def add_widget(self, widget, index=0, canvas=None):

        if issubclass(widget.__class__, AKSilverAppbarContent):
            self.ids.scroll_box.add_widget(widget)
        elif issubclass(widget.__class__, AKSilverAppbarHeader):
            self.ids.header.add_widget(widget)
        else:
            super().add_widget(widget, index=index, canvas=canvas)



from kivy.lang.builder import Builder
from kivymd.theming import ThemableBehavior
from kivy.uix.boxlayout import BoxLayout
from kivy.clock import Clock
from akivymd.uix.spinners import AKSpinnerDoubleBounce
from kivymd.uix.button import MDFillRoundFlatButton
from kivy.properties import ObjectProperty, NumericProperty, ListProperty, StringProperty
from kivy.animation import Animation

Builder.load_string(
    """
<Message_label@BoxLayout>:
    icon: ''
    text: ''
    padding: dp(4)
    MDIcon:
        size_hint_x: None
        width: dp(20)
        icon: root.icon
        theme_text_color: "Custom"
        text_color: 1,1,1,1
        halign: 'center'
        valign: 'center'

    MDLabel:
        text: root.text
        halign: 'center'
        valign: 'center'
        font_style: 'Caption'
        theme_text_color: "Custom"
        text_color: 1,1,1,1

<AKProgressbutton>:
    size_hint: None,None 
    size: self.minimum_size

    FloatLayout:
        id: float_box
        size_hint: None, None 

        BoxLayout: ## Success
            id: _success_box
            pos_hint: {'center_x': .5, 'center_y': .5}

            canvas.before:
                Color:
                    rgba: root.success_color
                    a: root._success_opacity
                RoundedRectangle:
                    size: root._success_box_size
                    pos: [ self.x+self.width/2-root._success_box_size[0]/2 , self.y ]
                    radius: [self.height / 2]

            Message_label:
                id: _success_label
                opacity: 0
                icon: root.success_icon
                text: root.success_text
                
        BoxLayout: ## Failure
            id: _failure_box
            pos_hint: {'center_x': .5, 'center_y': .5}

            canvas.before:
                Color:
                    rgba: root.failure_color
                    a: root._failure_opacity
                RoundedRectangle:
                    size: root._failure_box_size
                    pos: [ self.x+self.width/2-root._failure_box_size[0]/2 , self.y ]
                    radius: [self.height / 2]

            Message_label:
                id: _failure_label
                opacity: 0
                icon: root.failure_icon
                text: root.failure_text
    """
)

class AKProgressbutton(BoxLayout):
    
    success_icon= StringProperty('check')
    success_text= StringProperty('Success')
    success_color= ListProperty([0,0.7,0,1])

    failure_icon= StringProperty('close')
    failure_text= StringProperty('Failed')
    failure_color= ListProperty([1,0,0,1])

    duration= NumericProperty(0.2)
    animation = StringProperty('out_quad')

    _success_box_size= ListProperty([0,0])
    _failure_box_size= ListProperty([0,0])
    _success_opacity= NumericProperty(0)
    _failure_opacity=NumericProperty(0)

    reset_timeout= NumericProperty(2)

    def __init__(self,button=None,spinner=None, **kwargs):
        super().__init__(**kwargs) 
        Clock.schedule_once(lambda x: self._update())
        self.button= button
        self.spinner= spinner

    def _update(self):  
        
        if not self.button:
            self.button= MDFillRoundFlatButton(text="Ok")

        if not self.spinner:
            self.spinner= AKSpinnerDoubleBounce()
            
        self.button.pos_hint= {'center_x': .5, 'center_y': .5}
        self.button.bind(on_release=self._submit)

        self.spinner.pos_hint= {'center_x': .5, 'center_y': .5}

        self.ids.float_box.add_widget(self.button)
        self.ids.float_box.add_widget(self.spinner)

        self.ids.float_box.size = self.button.size
        self.spinner.spinner_size= self.button.height

        self._success_box_size= [0,self.button.height ]
        self._failure_box_size= [0,self.button.height ]

    def _spinner_state(self,state):
        self.spinner.active= state

    def _submit(self, instance):
        self._spinner_state(True)
        self._hide_button()

    def _hide_button(self,*args):
        anim = Animation(opacity=0, duration=self.duration, t=self.animation)
        anim.start(self.button )
        self.button.disabled= True

    def success(self):
        self._spinner_state(False)
        anim_box = Animation(_success_opacity=1,_success_box_size=self.button.size, duration=self.duration, t=self.animation)
        anim_label = Animation(opacity=1, duration=self.duration, t=self.animation)
        
        anim_box.start(self)
        anim_label.start(self.ids._success_label)

        Clock.schedule_once(lambda x: self._reset(), self.reset_timeout+self.duration)

    def failure(self):
        self._spinner_state(False)
        anim_box = Animation(_failure_opacity=1,_failure_box_size=self.button.size, duration=self.duration, t=self.animation)
        anim_label = Animation(opacity=1, duration=self.duration, t=self.animation)
        
        anim_box.start(self)
        anim_label.start(self.ids._failure_label)

        Clock.schedule_once(lambda x: self._reset(), self.reset_timeout+self.duration)

    def _reset(self):
        self.button.disabled= False
        self._spinner_state(False)
        button_anim = Animation(opacity=1, duration=self.duration, t=self.animation)

        success_box = Animation(_success_opacity=0,_success_box_size=[0,self.button.height], duration=self.duration, t=self.animation)
        success_label = Animation(opacity=0, duration=self.duration, t=self.animation)

        failure_box = Animation(_failure_opacity=0,_failure_box_size=[0,self.button.height], duration=self.duration, t=self.animation)
        failure_label = Animation(opacity=0, duration=self.duration, t=self.animation)
        
        button_anim.start(self.button )
        success_box.start(self)
        success_label.start(self.ids._success_label)
        failure_box.start(self)
        failure_label.start(self.ids._failure_label)



from kivy.lang.builder import Builder
from kivymd.app import MDApp
from kivy.uix.boxlayout import BoxLayout
from kivy.properties import ListProperty, NumericProperty,StringProperty,BooleanProperty,OptionProperty
from kivy.clock import Clock
from kivy.graphics import Ellipse, Color, Rotate, PushMatrix,PopMatrix
from kivymd.theming import ThemableBehavior
from kivymd.color_definitions import palette, colors
from kivy.utils import get_color_from_hex
from kivy.animation import Animation
from kivymd.uix.label import MDLabel
from akivymd.helper import point_on_circle
from kivy.core.window import Window 

'''issues
color_mode
'''

Builder.load_string(
    """
<PieChartNumberLabel>
    size_hint: None, None
    size: dp(40), dp(30)
    text: '%s\\n%d%%'%(root.title,root.percent)
    font_size: dp(10)
    halign: 'center'
    valign: 'center'
    font_style: 'Caption'
    theme_text_color: 'Custom'
    text_color: 1,1,1,1

<AKPieChart>:

    """
)

class PieChartNumberLabel(MDLabel):
    percent= NumericProperty(0)
    title= StringProperty('')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda x: self._update())
    
    def _update(self):
        self.x -= self.width/2
        self.y -= self.height/2

class AKPieChart(ThemableBehavior, BoxLayout):
    items= ListProperty()
    order= BooleanProperty(True)
    starting_animation= BooleanProperty(True)
    transition= StringProperty('out_cubic')
    duration= NumericProperty(1)
    color_mode= OptionProperty('colors', options=['primary_color', 'accent_color']) # not solved 

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
            
    def _format_items(self,items):
        percentage_sum= 0
        for k,v in items[0].items():
            percentage_sum+= v 

        if percentage_sum!=100:
            raise Exception('Sum of percenages must be 100')
        
        new_items={}
        for k,v in items[0].items():
            new_items[k]= 360*v/100
        
        if self.order:
            new_items= {k: v for k, v in sorted(new_items.items(), key=lambda item: item[1])}

        return new_items

    def _make_chart(self,items):
        self.size= (min(self.size) , min(self.size))
        if not items:
            raise Exception('Items cannot be empty.') 

        items= self._format_items(items)
        angle_start=0
        color_item= 0
        i= 1
        circle_center= [self.pos[0]+self.size[0]/2, self.pos[1]+self.size[1]/2]

        for title,value in items.items():
            with self.canvas.before:

                if self.starting_animation:
                    alpha= 0
                else:
                    alpha=1

                if self.color_mode=='colors':
                    color= get_color_from_hex(colors[palette[color_item]]['500']) 

                c= Color(rgb= color, a=alpha)
                if self.starting_animation:
                    e= Ellipse(pos=self.pos , size=self.size ,angle_start=angle_start, angle_end=angle_start+0.01)                    
                    
                    anim= Animation(size=self.size, angle_end=angle_start+value, duration=self.duration, t=self.transition)
                    anim_opcity= Animation(a=1, duration=self.duration*0.5)

                    anim_opcity.start(c)
                    anim.start(e)
                else: 
                    Ellipse(pos=self.pos , size=self.size ,angle_start=angle_start, angle_end=angle_start+value)
            color_item+=1
            angle_start+= value
                
        angle_start=0
        for title,value in items.items():
            with self.canvas.after:
                label_pos= point_on_circle( (angle_start+angle_start+value)/2 , circle_center, self.size[0]/3 )
                l= PieChartNumberLabel( x=label_pos[0], y= label_pos[1] , title=title)
                anim_label= Animation(percent=value*100/360)
                anim_label.start(l)
            angle_start+= value

    def _clear_canvas(self):
        try:
            self.canvas.before.clear()
            self.canvas.after.clear()
        except:
            pass 
    
    def on_pos(self,*args):
        self._clear_canvas()
        Clock.schedule_once(lambda x:self._make_chart(self.items) )

    def on_items(self, *args):
        self._clear_canvas()
        Clock.schedule_once(lambda x:self._make_chart(self.items) )
        



from kivy.uix.boxlayout import BoxLayout
from kivy.lang.builder import Builder
from kivy.properties import ListProperty, StringProperty, NumericProperty, BooleanProperty
from kivy.uix.behaviors import ButtonBehavior
from kivy.clock import Clock

from kivymd.uix.behaviors import MagicBehavior
from kivymd.theming import ThemableBehavior

Builder.load_string(
    """
<AKFloatingRoundedAppbarItemBase>:
    orientation: 'vertical'
    size_hint: None, None 
    height: self.parent.height- dp(5)
    width: self.minimum_width
    pos_hint: {'center_x': .5, 'center_y': .5}

<AKFloatingRoundedAppbarButtonItem>:
    MDIcon:
        icon: root.icon
        halign: 'center'
        valign: 'center'
        theme_text_color: 'Custom'
        text_color: root.icon_color if root.icon_color else 1,1,1,1
        font_size: dp(20)
        pos_hint: {'center_x': .5, 'center_y': .5}
        size_hint: None, None 
        size: self.font_size, self.font_size 

    Label:
        text: root.text
        halign: 'center'
        valign: 'center'
        font_size: dp(10)
        color: root.text_color if root.text_color else 1,1,1,1
        size_hint: None,None 
        size: self.texture_size 

<AKFloatingRoundedAppbarAvatarItem>:
    BoxLayout:
        size_hint: None,None 
        size : [self.parent.height,self.parent.height]
        pos_hint: {'center_x': .5, 'center_y': .5}
        canvas.after:
            Color: 
                rgba: 1,1,1,1
            Ellipse: 
                pos: self.pos 
                size: self.size 
                source: root.source

<AKFloatingRoundedAppbar>:
    size_hint: None,None 
    size: self.minimum_width, dp(40)
    pos_hint: {'center_x': .5}
    y: dp(10)
    spacing: dp(40)
    padding: dp(40)
    canvas.before:
        Color:
            rgba: root.bg_color if root.bg_color else root.theme_cls.primary_color
        RoundedRectangle:
            pos: self.pos 
            size: self.size 
            radius: [root.height/2,]

    """
)

class AKFloatingRoundedAppbarItemBase(ThemableBehavior, ButtonBehavior, MagicBehavior, BoxLayout):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda x: self._update() )

    def _update(self):
        if self.parent.press_effect:
            self.bind(on_press=lambda x: self.grow() )

class AKFloatingRoundedAppbarButtonItem(AKFloatingRoundedAppbarItemBase):
    text_color= ListProperty() 
    icon_color= ListProperty()
    icon= StringProperty()
    text= StringProperty()

class AKFloatingRoundedAppbarAvatarItem(AKFloatingRoundedAppbarItemBase):
    source= StringProperty() 

class AKFloatingRoundedAppbar(ThemableBehavior, BoxLayout):
    bg_color= ListProperty()
    press_effect= BooleanProperty(True)



from kivy.lang import Builder
from kivy.uix.modalview import ModalView
from kivy.uix.behaviors import ButtonBehavior
from kivymd.uix.behaviors import RectangularRippleBehavior
from kivymd.uix.dialog import BaseDialog
from kivy.uix.boxlayout import BoxLayout
from kivy.properties import StringProperty , NumericProperty , BooleanProperty , ListProperty, OptionProperty
from datetime import timedelta, datetime
from kivymd.theming import ThemableBehavior

Builder.load_string(
'''
<MDLabeltitle2@MDLabel>:
    theme_text_color: 'Custom'
    text_color: 1,1,1,1
    halign: 'center'
    vlighn: 'center'
    fon_style: 'H5'

<MDLabeltitle@MDLabel>
    theme_text_color: 'Primary'
    halign: 'center'
    vlighn: 'center'
    fon_style: 'Caption'   

<ButtonBase>
    size_hint_y: None
    height: dp(40)
    MDLabel:
        id: value
        text: root.text
        theme_text_color: 'Primary'
        halign: 'center'
        vlighn: 'center'

<AKDatePicker>:
    size_hint: None,None
    size: 
        (dp(302), dp(450))\
        if root.theme_cls.device_orientation == 'portrait'\
        else (dp(450) , dp(350))
        
    BoxLayout:
        orientation: 'vertical'
        canvas.before:
            Color:
                rgba: root.theme_cls.bg_normal
            RoundedRectangle:
                size: self.size 
                pos: self.pos    

        BoxLayout:
            size_hint_y: None
            height: dp(50)
            canvas.before:
                Color:
                    rgba: root.theme_cls.primary_color
                RoundedRectangle:
                    size: self.size 
                    pos: self.pos 
                    radius:[(10.0, 10.0), (10.0, 10.0), (0, 0), (0, 0)]
            
            MDLabeltitle2:
                text: root._year_title
            
            MDLabeltitle2:
                text: root._month_title

            MDLabeltitle2:
                text: root._day_title

        BoxLayout:
            size_hint_y: None
            height: dp(50)
            canvas.before:
                Color:
                    rgba: root.theme_cls.bg_dark
                Rectangle:
                    size: self.size 
                    pos: self.pos    

            MDLabeltitle:
                text: 'Year'
            
            MDLabeltitle:
                text: 'Month'
                                     
            MDLabeltitle:
                text: 'Day'


        BoxLayout:
            ScrollView:
                MDBoxLayout:
                    id: year_view
                    orientation: 'vertical'
                    adaptive_height: True
            ScrollView:
                MDBoxLayout:
                    id: month_view
                    orientation: 'vertical'
                    adaptive_height: True
            ScrollView:
                MDBoxLayout:
                    id: day_view
                    orientation: 'vertical'
                    adaptive_height: True
        BoxLayout:
            size_hint_y: None
            height: dp(40)
            padding: [dp(10) , 0]
            spacing: dp(10)
            canvas.before:
                Color:
                    rgba: root.theme_cls.bg_dark
                RoundedRectangle:
                    size: self.size 
                    pos: self.pos
                    radius: [(0.0, 10.0), (0.0, 10.0), (10, 10), (10, 10)]   
                             
            MDFlatButton:
                text: 'Cancel'
                pos_hint: {'center_x': .5 , 'center_y': .5}
                on_release: root.cancel()
                
            MDFlatButton:
                text: 'Select'
                pos_hint: {'center_x': .5 , 'center_y': .5}
                on_release: root._choose()
'''
)                    

class AKDatePicker(BaseDialog, ThemableBehavior):

    year_range= ListProperty([1930,2021])
    month_type= OptionProperty('string', options=['string', 'int'])
    _day_title = StringProperty('-')
    _month_title = StringProperty('-')
    _year_title = StringProperty('-')
    def __init__(self,callback = None , **kwargs):
        super(AKDatePicker , self).__init__(**kwargs)
        self.month_dic={
            '1': 'January',
            '2': 'February',
            '3': 'March',
            '4': 'April',
            '5': 'May',
            '6': 'June',
            '7': 'July',
            '8': 'August',
            '9': 'September',
            '10': 'October',
            '11': 'November',
            '12': 'December'
        }

        self.callback = callback
        for x in reversed(range(self.year_range[0], self.year_range[1])):
            self.ids.year_view.add_widget(ButtonBase(text='%d'%x, on_release= self._set_year))
        for x in reversed(range(1,13)):
            if self.month_type=='string':
                month= self.month_dic[str(x)]
            else: 
                month= str(x)

            self.ids.month_view.add_widget(ButtonBase(text=month, on_release=self._set_month))
        for x in reversed(range(1,32)):
            self.ids.day_view.add_widget(ButtonBase(text='%d'%x, on_release=self._set_day))

    def _set_day(self, instance):
        self._day_title= instance.text 
    
    def _set_month(self, instance):
        self._month_title= instance.text 

    def _set_year(self, instance):
        self._year_title= instance.text 

    def on_dismiss(self):
        self._year_title='-'
        self._month_title = '-'
        self._day_title= '-'
        return 

    def _choose(self):
        if not self.callback:
            return False

        if self.month_type=='string':
            for k,v in self.month_dic.items():
                if v==self._month_title:
                    self._month_title= k 
                    break

        try:
            date= datetime(
                int(self._year_title),
                int(self._month_title),
                int(self._day_title)
                )
        except:
            self.cancel()
            self.callback(False)

        self.callback(date)
        self.cancel()

    def cancel(self):
        self.dismiss()

class ButtonBase(RectangularRippleBehavior,ButtonBehavior,BoxLayout):
    text= StringProperty()



from kivy.properties import ListProperty, NumericProperty, StringProperty, OptionProperty
from kivy.animation import Animation
from kivymd.utils import asynckivy

class AKAddWidgetAnimationBehavior:
    items= ListProperty()
    speed= NumericProperty(0.05)
    animation_duration= NumericProperty(0.15)
    transition= OptionProperty('fade_size', options=['fade', 'fade_size', 'size'])
    direction= OptionProperty('vertical', options=['horizontal', 'vertical'])
    animation= StringProperty('out_quad')

    def on_items(self,*args):
        self.clear_widgets()
        async def add_item():
            for x in self.items:
                
                if 'fade' in self.transition:
                    x.opacity= 0 
                    anim= Animation(opacity=1, duration= self.animation_duration, t= self.animation)

                if 'size' in self.transition:
                    if self.direction=='horizontal':
                        current_width= x.width
                        x.size_hint_x= None 
                        x.width= current_width/2
                        anim= Animation(width= current_width, duration= self.animation_duration, t= self.animation)
                        anim &= Animation(opacity=1, duration= self.animation_duration*3, t= self.animation)

                    elif self.direction=='vertical':
                        current_height= x.height
                        x.size_hint_y= None 
                        x.height= current_height/2
                        anim= Animation(height= current_height, duration= self.animation_duration, t= self.animation)
                        anim &= Animation(opacity=1, duration= self.animation_duration*3, t= self.animation)
       
                anim.start(x)
                self.add_widget(x)
                await asynckivy.sleep(self.speed)
        return asynckivy.start(add_item())



from kivy.lang import Builder
from kivy.uix.image import AsyncImage
from kivy.clock import Clock
from kivy.properties import NumericProperty , BooleanProperty, StringProperty
from kivy.animation import Animation
from kivymd.theming import ThemableBehavior

__all__=('AKImageLoader')
Builder.load_string(
"""
<AKImageLoader>:
    source: ' '
    canvas.before:
        Color:
            rgba: root.theme_cls.bg_darkest
            a: root.fr_rec_opacity
        RoundedRectangle:
            pos: self.pos
            size: self.size
            radius: [(self.size[0]/2,self.size[1]/2) , (self.size[0]/2,self.size[1]/2),(self.size[0]/2,self.size[1]/2),(self.size[0]/2,self.size[1]/2) ] if root.circle==True else [(dp(20),dp(20)) , (dp(20),dp(20)) , (dp(20),dp(20)) , (dp(20),dp(20)) ]
        Color:
            rgba: root.theme_cls.bg_dark   
            a: root.bg_rec_opacity
        RoundedRectangle:
            pos: self.pos
            size: self.size
            radius: [(self.size[0]/2,self.size[1]/2) , (self.size[0]/2,self.size[1]/2),(self.size[0]/2,self.size[1]/2),(self.size[0]/2,self.size[1]/2) ] if root.circle==True else [(dp(20),dp(20)) , (dp(20),dp(20)) , (dp(20),dp(20)) , (dp(20),dp(20)) ]

"""
)

class AKImageLoader(ThemableBehavior, AsyncImage):

    bg_rec_opacity= NumericProperty(0)
    fr_rec_opacity= NumericProperty(0)
    circle= BooleanProperty(True)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.start_anim = None 

    def _check_source(self,source):
        if source==False or len(source.strip())==0 :
            self.source=' '
            return False 
        else:
            return True 

    def _start_animate(self):
        self.bg_rec_opacity=1 
        self.fr_rec_opacity=1
        self.color = [1,1,1,0]
        duration= 0.8

        self.start_anim = Animation(bg_rec_opacity=1 , t='in_quad' , duration=duration )\
            + Animation(bg_rec_opacity=0, t='out_quad', duration=duration )
        self.start_anim.repeat = True 
        self.start_anim.start(self)

    def _stop_animate(self):
        duration= 0.8

        self.color=[1,1,1,1]
        if self.start_anim:
            self.start_anim.cancel_all(self)
            self.stop_anim = Animation(fr_rec_opacity=0 , t='out_quad' , duration=0.1 )
            self.stop_anim &= Animation(bg_rec_opacity=0 , t='out_quad' , duration=0.1 )
            self.stop_anim.start(self)

    def on_source(self,*args):
        if self._check_source(self.source):
            self._stop_animate() 
        else: 
            self._start_animate() 


    


from kivy.lang.builder import Builder 
from kivy.properties import NumericProperty, StringProperty, BooleanProperty
from kivy.animation import Animation

import time
Builder.load_string(
    """
<AKAnimationBehaviorBase>:
    canvas.before:
        PushMatrix
        Rotate:
            angle: root._angle
            origin: self.center     

    canvas.after:
        PopMatrix           

    """
)

class AKAnimationBehaviorBase:
    
    duartion = NumericProperty(0.5)
    transition= StringProperty('out_cubic')
    animation_disabled= BooleanProperty(False)
    
    _angle= NumericProperty()
    _first_text= True 

    def _start_animate(self):
        if self.animation_disabled:
            return 

        if self._angle<=180:
            _angle= 360
        else:
            _angle= 0

        if not self._first_text:
            anim= Animation(_angle=_angle, duration= self.duartion, t=self.transition)
            anim.start(self)
        
        self._first_text= False 

class AKAnimationTextBehavior(AKAnimationBehaviorBase):

    def on_text(self,*args):
        self._start_animate()

class AKAnimationIconBehavior(AKAnimationBehaviorBase):
    
    def on_icon(self, *args):
        self._start_animate()



from kivy.lang import Builder
from kivymd.uix.label import MDLabel
from kivy.clock import Clock
from kivy.properties import NumericProperty
from kivy.animation import Animation

__all__=('AKLabelLoader')

Builder.load_string(
"""

<AKLabelLoader>:
    canvas.before:
        Color:
            rgba: root.theme_cls.bg_darkest
            a: root.fr_rec_opacity
        RoundedRectangle:
            pos: self.pos
            size: self.size
            radius: [(dp(20),dp(20)) , (dp(20),dp(20)),(dp(20),dp(20)),(dp(20),dp(20)) ]
        Color:
            rgba: root.theme_cls.bg_dark   
            a: root.bg_rec_opacity
        RoundedRectangle:
            pos: self.pos
            size: self.size
            radius: [(dp(20),dp(20)) , (dp(20),dp(20)),(dp(20),dp(20)),(dp(20),dp(20)) ]

"""
)


class AKLabelLoader(MDLabel):

    bg_rec_opacity= NumericProperty(0)
    fr_rec_opacity= NumericProperty(0)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.start_anim = None 
        Clock.schedule_once(lambda x: self._update(self.text))

    def _update(self,text):
        if self._check_text(text):
            self._stop_animate() 
        else: 
            self._start_animate() 

    def _check_text(self,text):
        if not text :
            return False 
        else:
            return True 

    def _start_animate(self):
        self.bg_rec_opacity=1 
        self.fr_rec_opacity=1
        duration= 0.8
        self.start_anim = Animation(bg_rec_opacity=1 , t='in_quad' , duration=duration )\
            + Animation(bg_rec_opacity=0, t='out_quad', duration=duration )
        self.start_anim.repeat = True 
        self.start_anim.start(self)

    def _stop_animate(self):
        duration= 0.8

        if self.start_anim:
            self.start_anim.cancel_all(self)

        if self.bg_rec_opacity!=0 and self.fr_rec_opacity!=0:
            self.stop_anim = Animation(fr_rec_opacity=0 , t='out_quad' , duration=duration )
            self.stop_anim &= Animation(bg_rec_opacity=0 , t='out_quad' , duration=duration )
            self.stop_anim.start(self)

    def on_text(self,*args):
        if self._check_text(self.text):
            self._stop_animate() 
        else: 
            self._start_animate() 

    


from kivy.lang import Builder
from kivy.uix.boxlayout import BoxLayout 
from kivy.uix.stacklayout import StackLayout
from kivy.uix.behaviors import ButtonBehavior
from kivy.properties import NumericProperty , StringProperty ,BooleanProperty, ListProperty
from kivy.metrics import dp
from kivy.animation import Animation
from kivy.uix.image import Image
from kivymd.uix.behaviors.ripplebehavior import CircularRippleBehavior
from kivymd.theming import ThemableBehavior

Builder.load_string(
    """
<AKSelectList>
    orientation: 'lr-tb'    
    spacing: dp(5)
    padding: dp(10)
    size_hint_y: None
    height: self.minimum_height

<AKSelectListAvatarItem>
    orientation: 'vertical'
    size_hint: 1/root.columns, None
    height: self.width*1.2
    padding: dp(5)
    spacing: dp(5)
    on_release: root._choose_selection(_first_label.text) 

    FloatLayout:
        Image:
            pos_hint: {'center_x': .5 , 'center_y': .5}
            keep_ratio: True
            source: root.source

        MDIcon:     
            id: _box
            pos_hint: {'center_x': 0.9 , 'center_y': 0.9}
            size_hint: None,None
            font_size: 0
            icon: 'check-circle'
            color: root.theme_cls.primary_color
            
    BoxLayout:
        size_hint_y: None
        height: dp(50)
        orientation: 'vertical'
        spacing: dp(4)

        MDLabel:
            id: _first_label
            text: root.first_label
            theme_text_color: 'Primary'
            halign: 'center'

        MDLabel:
            text: root.second_label
            halign: 'center'
            theme_text_color: 'Secondary'
            font_style: 'Caption'
    """
)

class AKSelectListAvatarItem(ThemableBehavior,ButtonBehavior,CircularRippleBehavior,BoxLayout):
    columns = NumericProperty(4)
    source = StringProperty('')
    first_label = StringProperty('')
    second_label= StringProperty('')
    animate_start= BooleanProperty(True)

    def __init__(self,**kwargs):
        super().__init__(**kwargs)

    def _choose_selection(self , select):
        
        selected_list = self.parent._selected_list

        if select not in selected_list:
            selected_list.append(select)
            self._selection_anim()

        else:
            selected_list.remove(select)
            self._deselection_anim()

        if not selected_list:
            selected_list  = []

        self.parent._selected_list = selected_list
    
    def _selection_anim(self):
            anim = Animation(font_size=self.width/3 , t='out_bounce' , duration=0.1) 
            anim.start(self.ids._box)

    def _deselection_anim(self):
            anim = Animation(font_size=0 ,size=self.ids._box.texture_size, t='in_bounce' , duration=0.1)
            anim.start(self.ids._box)

class AKSelectList(StackLayout):
    _selected_list = []

    def get_selection(self):
        return self._selected_list

    def clear_selection(self):
        if not self.children:
            return

        for child in self.children:
            if child.first_label in self._selected_list:
                child._deselection_anim()
        self._selected_list= []

    def select_all(self):
        for child in self.children:
            if child.first_label not in self._selected_list:
                child._selection_anim()
                self._selected_list.append(child.first_label)



from math import cos, sin , radians  

def point_on_circle(degree , center , dis  ):
    '''
        Finding the x,y coordinates on circle, based on given angle
    '''

    if 0<=degree <=90:
        degree= 90- degree
    elif 90<degree<360:
        degree= 450-degree

    radius = dis
    angle = radians(degree)

    x = center[0] + (radius * cos(angle))
    y = center[1] + (radius * sin(angle))

    return [x,y]



from kivy.logger import Logger

__version__ = 1.1
__description__ = 'A set of fancy widgets for KivyMD'
__author__= "Sina Namadian"
__email__="quitegreensky@gmail.com"
Logger.info(f"AKivymd: v{__version__}")

import akivymd.factory_registers
from akivymd.tools.pyinstaller_hooks import hooks_path


from kivy.utils import platform
from kivy.utils import get_hex_from_color

def change_statusbar_color(statuscolor, icons_color='Light'):

    if platform!='android':
        return

    from android.runnable import run_on_ui_thread
    from jnius import autoclass

    Color = autoclass("android.graphics.Color")
    WindowManager = autoclass('android.view.WindowManager$LayoutParams')
    activity = autoclass('org.kivy.android.PythonActivity').mActivity
    View= autoclass('android.view.View')

    def statusbar(*args):
        color= get_hex_from_color(statuscolor)[:7]
        window = activity.getWindow()

        if icons_color=='Dark':
            window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR)
        elif icons_color=='Light':
            window.getDecorView().setSystemUiVisibility(0)

        window.clearFlags(WindowManager.FLAG_TRANSLUCENT_STATUS)
        window.addFlags(WindowManager.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.setStatusBarColor(Color.parseColor(color)) 
        window.setNavigationBarColor(Color.parseColor(color))


    status= run_on_ui_thread(statusbar)
    
    return status()


from kivy.lang import Builder
from kivy.clock import Clock
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.carousel import Carousel
from kivy.uix.widget import Widget
from kivymd.theming import ThemableBehavior
from kivy.properties import  NumericProperty, BooleanProperty, StringProperty, ListProperty
from kivy.core.window import Window 
from kivy.animation import Animation
from kivy.event import EventDispatcher
from kivy.metrics import dp

Builder.load_string(
    """
<ItemCircles>:
    size_hint_x: None 
    canvas.before:
        Color:
            rgba: root._circles_color
        Line:  
            circle: [ self.pos[0]+self.width/2, self.pos[1]+self.height/2, self.width/2]
            width: dp(1)

<AKOnboardingItem>:

<AKOnboarding>: 
    orientation: 'vertical'

    MyCarousel:
        min_move:root.min_move
        anim_type: root.anim_type
        anim_move_duration: root.anim_move_duration
        id: carousel

    FloatLayout:
        id: rounded_box
        size_hint_y: None 
        height: circles_box.y+ circles_box.height*2
        canvas.before:
            Color:
                rgba: root.bottom_bar_color if root.bottom_bar_color else app.theme_cls.bg_dark
                a: 1 if root.show_bottom_bar else 0
            RoundedRectangle:
                pos: self.pos
                size: self.size   
                radius: root.bottom_bar_radius

        Widget:
            id: ghost_circle     
            size_hint: None, None 
            canvas.before:
                Color:
                    rgba: root.circles_color if root.circles_color else root.theme_cls.primary_color
                Ellipse:  
                    pos: self.pos
                    size: self.size 

        BoxLayout:
            id: circles_box
            pos: rounded_box.width/2-self.width/2 , rounded_box.height/2-self.height/2
            size_hint: None,None 
            size: self.minimum_width , root.circles_size
            spacing: root.circles_size/2

        MDFlatButton:
            text: 'Skip'
            on_release: root._on_finish_dispatch()
            disabled: False if root.skip_button else True
            opacity: 1 if root.skip_button else 0
            text_color: root.circles_color if root.circles_color else root.theme_cls.primary_color
            pos_hint: {'right': .95, 'center_y': .5}

    """
)


class ItemCircles(ThemableBehavior, Widget):
    _circles_color= ListProperty(None) 

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

class MyCarousel(ThemableBehavior,Carousel):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda x: self._add_circles())
        Window.bind(on_resize=self._on_resize)

    def _add_circles(self):
        self.total_circles= len(self.slides)-1

        if self.parent.circles_color:
            circle_color= self.parent.circles_color
        else:
            circle_color= self.theme_cls.primary_color

        for _ in range(self.total_circles+1):
            self.parent.ids.circles_box.add_widget(ItemCircles(width= self.parent.circles_size, _circles_color=circle_color))

        self._current_circle= self.total_circles
        Clock.schedule_once(lambda x: self._set_current_circle(animation=False))

    def on_size(self, *args):
        Clock.schedule_once(lambda x:self._set_current_circle(animation=False))
        return super().on_size(*args)

    def reset(self):
        self._current_circle= self.total_circles
        self._set_current_circle()
        self.load_slide(self.slides[0])

    def _set_current_circle(self, mode=None, animation=True):
        if mode=='next':
            if self._current_circle>0:
                self._current_circle -=1
            else:
                self.parent._on_finish_dispatch()

        elif mode=='previous':
            if self._current_circle<self.total_circles:
                self._current_circle +=1 
        if animation:
            width= self.parent.ids.ghost_circle.width
            anim= Animation(
                pos=self.parent.ids.circles_box.children[self._current_circle].pos, 
                t= self.anim_type, 
                duration= self.anim_move_duration
                )
            anim.start(self.parent.ids.ghost_circle)
        else:
            self.parent.ids.ghost_circle.pos= self.parent.ids.circles_box.children[self._current_circle].pos

    def on_touch_up(self, touch):
        if abs(self._offset)> self.width*self.min_move:

            if self._offset>0:  ## previous screen
                self._set_current_circle('previous')

            elif self._offset<0:  ## next screen
                self._set_current_circle('next')
        
        return super().on_touch_up(touch)

    def _on_resize(self,*args):
        Clock.schedule_once(lambda x: self._set_current_circle(animation=False))

class AKOnboardingItem(BoxLayout):
    pass 

class AKOnboarding(ThemableBehavior,BoxLayout,EventDispatcher):

    circles_size= NumericProperty(dp(20))
    skip_button= BooleanProperty(True)
    min_move= NumericProperty(0.05)
    anim_type= StringProperty('out_quad')
    anim_move_duration= NumericProperty(0.2)
    bottom_bar_radius= ListProperty([dp(20), dp(20),0 ,0])
    show_bottom_bar= BooleanProperty(True)
    bottom_bar_color= ListProperty(None)
    circles_color= ListProperty(None)

    def __init__(self, **kwargs):
        super(AKOnboarding, self).__init__(**kwargs)
        self.register_event_type('on_finish')
        Clock.schedule_once(lambda x: self._update())

    def add_widget(self, widget, index=0, canvas=None):
        if issubclass(widget.__class__ ,AKOnboardingItem):
            self.ids.carousel.add_widget(widget)
        else:
            super().add_widget(widget, index=index, canvas=canvas)

    def _on_finish_dispatch(self):
        self.dispatch('on_finish')

    def on_finish(self,*args):
        pass 
    
    def reset(self):
        return self.ids.carousel.reset()

    def on_size(self,*args):
        self.ids.carousel.size= self.size 

    def _update(self):
        self.ids.ghost_circle.size= [self.circles_size, self.circles_size]



from kivy.lang.builder import Builder
from kivy.uix.widget import Widget
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.floatlayout import FloatLayout
from kivy.properties import NumericProperty, ListProperty, StringProperty, BooleanProperty, OptionProperty
from kivy.metrics import dp 
from kivy.clock import Clock
from kivymd.theming import ThemableBehavior

Builder.load_string(
    """
<BadgeItem>
    size_hint: None,None 
    padding: dp(5)
    size: self.size if root.text else [dp(20), dp(20)]
    opacity: 1 if self.badge_disabled==False else 0

    pos: 
        (self.parent.x  -self.width*(root.offset), self.parent.y+ self.parent.height  -self.height*(1-root.offset)) if root.position=='left'\
        else (self.parent.x +self.parent.width -self.width*(1-root.offset) , self.parent.y +self.parent.height -self.height*(1-root.offset)) 

    canvas.before:
        Color:
            rgba:  root.bg_color if root.bg_color else root.theme_cls.bg_normal  
        RoundedRectangle:
            pos:self.pos 
            size: self.size 
            radius: [root.height/2, root.height/2, root.height/2, root.height/2]

        Color:
            rgba: root.badgeitem_color if root.badgeitem_color else root.theme_cls.accent_color
        RoundedRectangle:
            pos: 
                [self.pos[0]+ root.badgeitem_padding/2  , self.pos[1]+ root.badgeitem_padding/2]
            size: [ self.size[0]-root.badgeitem_padding, self.size[1]-root.badgeitem_padding ]
            radius: [root.height/2, root.height/2, root.height/2, root.height/2]
    Label:
        size_hint: None,None 
        size: self.texture_size[0]+dp(10), self.texture_size[1]
        halign: 'center'
        valign: 'center'
        font_size: dp(13)
        bold: root.bold
        pos_hint: {'center_x': .5, 'center_y': .5}
        text: root.text
        color: 1,1,1,1

<AKBadgeLayout>:

    size_hint: None,None 
    size: box.size 
    BadgeContent:
        id: box
        pos: root.pos  
        size_hint: None,None 
        size: self.minimum_size 

    BadgeItem:
        id: badge
        size_hint: None,None 
        size: self.minimum_size
        bg_color: root.bg_color
        badgeitem_padding: root.badgeitem_padding
        badgeitem_color: root.badgeitem_color
        position: root.position
        text: root.text
        bold: root.bold
        offset: root.offset
        badge_disabled: root.badge_disabled

    """
)

class BadgeContent(BoxLayout):
    pass 

class BadgeItem(ThemableBehavior, BoxLayout):
    bg_color= ListProperty()
    badgeitem_padding= NumericProperty()
    badgeitem_color= ListProperty()
    position= OptionProperty('right', options=['right', 'left'])
    text= StringProperty('')
    bold= BooleanProperty()
    offset= NumericProperty()
    badge_disabled= BooleanProperty(False)
        
class AKBadgeLayout(FloatLayout):
    badgeitem_size= NumericProperty(dp(20))  
    bg_color= ListProperty()
    badgeitem_padding= NumericProperty(dp(3))
    badgeitem_color= ListProperty()
    position= StringProperty('right')
    text= StringProperty('')
    bold= BooleanProperty(False)
    offset= NumericProperty(0.25)
    badge_disabled= BooleanProperty(False)

    def add_widget(self, widget, index=0, canvas=None):

        if issubclass(widget.__class__, BadgeItem) or\
            issubclass(widget.__class__, BadgeContent):

            return super().add_widget(widget, index=index, canvas=canvas)
        else:
            return self.ids.box.add_widget(widget)



from kivy.lang import Builder
from kivy.uix.carousel import Carousel
from kivymd.uix.dialog import BaseDialog
from kivymd.theming import ThemableBehavior
from kivy.properties import ObjectProperty, NumericProperty, StringProperty, BooleanProperty
from kivy.uix.scatter import Scatter
from kivy.animation import Animation
from kivy.metrics import dp 

Builder.load_string(
    '''
<Navigation_button@MDIconButton>
    theme_text_color: "Custom"
    text_color: 1,1,1,1

<AKImageViewerItem>:
    id: scatter
    do_rotation: False
    Image:
        id: _image
        size_hint: None,None
        size: scatter.size
        source: root.source

<AKImageViewer>:
    id: _mainbox
    size_hint: (0.9, 0.4) if root.theme_cls.device_orientation == 'portrait' else (0.9,0.7)
    carousel: carousel

    FloatLayout:
 
        Navigation_button:
            id: _left_button
            pos_hint: {'x': 0 , 'center_y': .5}
            icon: 'chevron-left'
            on_release: 
                root._previous_slide()
                root._reset_current_slide()

        Navigation_button:
            id: _right_button
            icon: 'chevron-right'
            pos_hint: {'right': 1 , 'center_y': .5}
            on_release: 
                root._next_slide()
                root._reset_current_slide()

        Carousel:
            id: carousel
            size_hint: None,None
            size: _mainbox.size[0]-2*_left_button.size[0], _mainbox.size[1]
            pos_hint: {'center_x': .5 , 'center_y': .5}
            scroll_distance: dp(9999)
            min_move: 0
            
    '''
)

class AKImageViewerItem(Scatter):
    zoom_max= NumericProperty(2)
    bounce_duration= NumericProperty(0.3)
    bounce_animation= StringProperty('out_cubic')
    source= StringProperty()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.zoom_max_exceeded= False
        self.zoom_min_exceeded= False
        self.zoom_min= 1
        self.scale= self.zoom_min

    def on_touch_down(self, touch):
        if touch.is_double_tap:
            if self.scale==self.zoom_min:
                self.reset_max_zoom()
            elif self.zoom_max>=self.scale>self.zoom_min:
                self.reset_min_zoom()

        return super().on_touch_down(touch)


    def on_touch_up(self, touch):
        if self.zoom_max_exceeded:
            self.reset_max_zoom()
        
        if self.zoom_min_exceeded:
            self.reset_min_zoom()

        self.zoom_max_exceeded= False 
        self.zoom_min_exceeded= False 
        return super().on_touch_up(touch)

    def on_touch_move(self, touch):
        super().on_touch_move(touch)
        if self.scale> self.zoom_max:
            self.zoom_max_exceeded= True 
        else: 
            self.zoom_max_exceeded= False 

        if self.scale<self.zoom_min:
            self.zoom_min_exceeded= True 
        else: 
            self.zoom_min_exceeded= False 

    def reset_max_zoom(self):
        max_zoom_anim = Animation(scale=self.zoom_max , duration= self.bounce_duration, t=self.bounce_animation )
        max_zoom_anim.start(self)

    def reset_position(self):
        position_anim = Animation(pos=(0,0) , duration= self.bounce_duration, t=self.bounce_animation )
        position_anim.start(self)

    def reset_min_zoom(self):
        min_zoom_anim= Animation(scale= self.zoom_min, duration= self.bounce_duration, t=self.bounce_animation )
        min_zoom_anim.start(self)
        self.reset_position()

class AKImageViewer(BaseDialog):
    carousel= ObjectProperty(None)

    def __init__(self, **kwargs):
        super().__init__(**kwargs) 

    def _next_slide(self):
        if len(self.carousel.slides)==0:
            return
            
        if self.carousel.current_slide== self.carousel.slides[-1]:
            self.carousel.load_slide(self.carousel.slides[0])
        else:
            self.carousel.load_next(mode='next')

    def on_dismiss(self):
        self.carousel.load_slide(self.carousel.slides[0])
        self.carousel.current_slide.reset_min_zoom()
        self.carousel.current_slide.reset_position()

    def _previous_slide(self):
        if len(self.carousel.slides)==0:
            return

        if self.carousel.current_slide== self.carousel.slides[0]:
            self.carousel.load_slide(self.carousel.slides[-1])
        else:        
            self.carousel.load_previous()

    def _reset_current_slide(self):
        self.carousel.current_slide.reset_min_zoom()
        self.carousel.current_slide.reset_position()

    def add_widget(self, widget, index=0, canvas=None):
        if issubclass(widget.__class__, AKImageViewerItem):
            self.carousel.add_widget(widget) 
        else:
            return super().add_widget(widget, index=index, canvas=canvas)




from setuptools import setup, find_packages
import akivymd
requirements_path='requirements.txt'

with open("README.md", "r") as fh:
    long_description = fh.read()

def parse_requirements(filename):
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]

install_reqs = parse_requirements(requirements_path)
setup(
    name='akivymd',
    version=str(akivymd.__version__),
    description=akivymd.__description__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/quitegreensky/akivymd",
    author=akivymd.__author__,
    author_email=akivymd.__email__,
    license='MIT',
    install_requires=parse_requirements(requirements_path) ,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],    
      )


from kivy.lang.builder import Builder
from kivy.uix.widget import Widget
from kivy.properties import NumericProperty , ListProperty ,StringProperty,BooleanProperty
from kivymd.theming import ThemableBehavior
from kivy.animation import Animation
from kivy.clock import Clock
from kivy.metrics import dp
Builder.load_string("""
<AKSpinnerBase>:
    size_hint: None, None

<AKSpinnerCircleFlip>:
    size: root.spinner_size , root.spinner_size

    canvas:
        Color:
            rgba: root.color if root.color else root.theme_cls.primary_color
        Ellipse:
            size:  root._circle_size
            pos: [ self.x+self.width/2 - root._circle_size[0]/2  , self.y+self.height/2 - root._circle_size[1]/2 ]

<AKSpinnerDoubleBounce>:
    size: root.spinner_size , root.spinner_size

    canvas:
        Color:
            rgba: root.color if root.color else root.theme_cls.primary_color    
        Ellipse:
            size:  root._circle_size1
            pos: [ self.x+self.width/2 - root._circle_size1[0]/2  , self.y+self.height/2 - root._circle_size1[1]/2 ]
        
        Color:
            rgba: root.color_secondary if root.color_secondary else root.theme_cls.primary_light
        Ellipse:
            size:  root._circle_size2
            pos: [ self.x+self.width/2 - root._circle_size2[0]/2  , self.y+self.height/2 - root._circle_size2[1]/2 ]

<AKSpinnerFoldingCube>:
    size: root.spinner_size , root.spinner_size

    canvas:
        PushMatrix
        Rotate:
            axis: 0,0,1
            angle: root.angle
            origin: self.center    
        Color:
            rgba: root.color if root.color else root.theme_cls.primary_color
            a:root._cube1a
        Rectangle:
            size: root._cubeitem1
            pos: [self.x, self.y]
        Color:
            rgba: root.color if root.color else root.theme_cls.primary_color
            a:root._cube2a            
        Rectangle:
            size: root._cubeitem2
            pos: [self.x, self.y+self.height/2  ] 
        Color:
            rgba: root.color if root.color else root.theme_cls.primary_color
            a:root._cube3a            
        Rectangle:
            size: root._cubeitem3
            pos: [self.x+ self.width/2 , self.y+ self.height - root._cubeitem3[1]]
        Color:
            rgba: root.color if root.color else root.theme_cls.primary_color
            a:root._cube4a            
        Rectangle:
            size: root._cubeitem4
            pos: [self.x+ self.width - root._cubeitem4[0] , self.y]
        PopMatrix    

<AKSpinnerThreeDots>:
    size: root.spinner_size*3 , root.spinner_size
    BoxLayout:
        spacing: self.size[1]
        size_hint: None, None 
        size: root.size 
        pos: root.pos 

        Widget:
            canvas:
                Color:
                    rgba: root.color if root.color else root.theme_cls.primary_color    
                Ellipse:
                    size:  root._circle_size1
                    pos: [ self.x+self.width/2 - root._circle_size1[0]/2  , self.y+self.height/2 - root._circle_size1[1]/2 ]
        Widget:
            canvas:
                Color:
                    rgba: root.color if root.color else root.theme_cls.primary_color    
                Ellipse:
                    size:  root._circle_size2
                    pos: [ self.x+self.width/2 - root._circle_size2[0]/2  , self.y+self.height/2 - root._circle_size2[1]/2 ]

        Widget:
            canvas:
                Color:
                    rgba: root.color if root.color else root.theme_cls.primary_color    
                Ellipse:
                    size:  root._circle_size3
                    pos: [ self.x+self.width/2 - root._circle_size3[0]/2  , self.y+self.height/2 - root._circle_size3[1]/2 ]
                    
""")

class AKSpinnerBase(ThemableBehavior,Widget):
    spinner_size = NumericProperty(48) 
    speed = NumericProperty(0.4)
    active = BooleanProperty(False)  
    color= ListProperty()

class AKSpinnerCircleFlip(AKSpinnerBase):
    animation= StringProperty('out_back')

    _circle_size = ListProperty([0,0])

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        
    def _start_animate(self , size):
        size = [size,size]
        self.flip_v_start = Animation(_circle_size = [size[0] , size[1]],opacity = 1, duration = self.speed  , t=self.animation  )
        self.flip_v = Animation(_circle_size = [size[0] , 0], duration = self.speed  , t=self.animation  ) \
            + Animation(_circle_size = [size[0] , size[1]], duration = self.speed  , t=self.animation  )\
            + Animation(_circle_size = [0 , size[1]], duration = self.speed  , t=self.animation  )\
            + Animation(_circle_size = [size[0] , size[1]], duration = self.speed  , t=self.animation  )

        self.flip_v.repeat = True
        self.flip_v_start.start(self)
        Clock.schedule_once(lambda x:self.flip_v.start(self) , self.speed)

    def _stop_animate(self):
        self.flip_v_start.cancel_all(self)
        self.flip_v.cancel_all(self)
        self.flip_v_stop =  Animation(_circle_size = [0 , 0],opacity = 0, duration = 0.1 , t=self.animation  )
        self.flip_v_stop.start(self)

    def on_active(self,*args):
        size= self.size[0]
        if self.active:
            self._start_animate(size)
        else:
            self._stop_animate()

class AKSpinnerDoubleBounce(AKSpinnerBase):
    color_secondary= ListProperty()

    _circle_size1 = ListProperty([0,0])
    _circle_size2 = ListProperty([0,0])

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        
    def _update(self , size ):
        self._circle_size1 = [size,size]
        self._circle_size2 = [0,0]
        self._start_animate(size)

    def _start_animate(self , size):
        self.anim0 = Animation(_circle_size1=[size,size] ,opacity=1, t='out_quad' , duration=self.speed)

        self.anim1 = Animation(_circle_size1 = [size/2, size/2] , t='in_quad' , duration=self.speed)\
            + Animation(_circle_size1 = [size, size], t='out_quad', duration=self.speed)

        self.anim2 = Animation(_circle_size2 = [size/2, size/2] ,opacity=1, t='in_quad', duration=self.speed)\
            + Animation(_circle_size2 = [0, 0], t='out_quad', duration=self.speed)

        self.anim1.repeat= True
        self.anim2.repeat= True

        self.anim0.start(self)
        Clock.schedule_once(lambda x:self.anim1.start(self) , self.speed)
        Clock.schedule_once(lambda x:self.anim2.start(self) , self.speed)

    def _stop_animate(self):
        self.anim0.cancel_all(self)
        self.anim1.cancel_all(self) 
        self.anim2.cancel_all(self) 
        self.anim1_stop = Animation(_circle_size1 = [0, 0] ,opacity=0, t='in_quad' , duration=0.1)
        self.anim2_stop = Animation(_circle_size2 = [0, 0] ,opacity=0, t='in_quad' , duration=0.1)
        self.anim1_stop.start(self)
        self.anim2_stop.start(self)

    def on_active(self,*args):
        size= self.size[0]
        if self.active:
            self._start_animate(size)
        else:
            self._stop_animate()

class AKSpinnerFoldingCube(AKSpinnerBase):
    angle= NumericProperty(45)
    animation = StringProperty('out_cubic')

    _cubeitem1 = ListProperty([0,0])
    _cubeitem2 = ListProperty([0,0])
    _cubeitem3 = ListProperty([0,0])
    _cubeitem4 = ListProperty([0,0])
    _cube1a = NumericProperty(0)
    _cube2a = NumericProperty(0)
    _cube3a = NumericProperty(0)
    _cube4a = NumericProperty(0)
    
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _start_animate(self , size):
        size/=2
        self.cube_fold = Animation(_cubeitem1 = [size ,size] , _cube1a=1 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem2 = [size ,size] , _cube2a=1 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem3 = [size ,size], _cube3a=1 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem4 = [size ,size], _cube4a=1 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem4 = [0 ,size], _cube4a=0 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem3 = [size ,0], _cube3a=0 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem2 = [0 ,size] , _cube2a=0 , duration = self.speed, t=self.animation)\
            + Animation(_cubeitem1 = [size ,0] , _cube1a=0 , duration = self.speed, t=self.animation)\

        self.cube_fold.repeat = True
        self.cube_fold.start(self)

    def _update(self,size):
        self._cubeitem1 = [size/2,0 ]
        self._cubeitem2 = [0 , size/2]
        self._cubeitem3 = [size/2,0 ]
        self._cubeitem4 = [0 , size/2]

    def _stop_animate(self,size):
        size/=2
        self.cube_fold.cancel_all(self)
        self.cube_stop = Animation(_cubeitem4 = [0 ,size], _cube4a=0 , duration = 0.1, t=self.animation)\
            + Animation(_cubeitem3 = [size ,0], _cube3a=0 , duration = 0.1, t=self.animation)\
            + Animation(_cubeitem2 = [0 ,size] , _cube2a=0 , duration = 0.1, t=self.animation)\
            + Animation(_cubeitem1 = [size ,0] , _cube1a=0 , duration = 0.1, t=self.animation)\
        
        self.cube_stop.start(self)
        
    def on_active(self, *args):
        size = self.size[0]
        self._update(size)
        if self.active:
            self._start_animate(size) 
        else:
            self._stop_animate(size) 

class AKSpinnerThreeDots(AKSpinnerBase):

    animation = StringProperty('linear')

    _circle_size1 = ListProperty([0,0])
    _circle_size2 = ListProperty([0,0])
    _circle_size3 = ListProperty([0,0])

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _start_animate(self , size):
        self.anim1 = Animation(_circle_size1= [size,size],opacity= 1, duration=self.speed ,t=self.animation)\
            + Animation(_circle_size1= [0,0] , duration=self.speed,t=self.animation)\
            + Animation(duration=self.speed)\

        self.anim2 =Animation(_circle_size2= [size,size],opacity= 1, duration=self.speed,t=self.animation)\
            + Animation(_circle_size2= [0,0] , duration=self.speed,t=self.animation)\
            + Animation(duration=self.speed)\

        self.anim3 =Animation(_circle_size3= [size,size],opacity= 1, duration=self.speed,t=self.animation)\
            + Animation(_circle_size3= [0,0] , duration=self.speed,t=self.animation)\
            + Animation(duration=self.speed)\

        self.anim1.repeat = True 
        self.anim2.repeat = True 
        self.anim3.repeat = True 

        self.anim1.start(self)
        Clock.schedule_once(lambda dt: self.anim2.start(self) , self.speed)
        Clock.schedule_once(lambda dt: self.anim3.start(self), self.speed*2)

    def _stop_animate(self):
        self.anim1.cancel_all(self)
        self.anim2.cancel_all(self)
        self.anim3.cancel_all(self)
        self.anim1_stop = Animation(_circle_size1= [0,0] ,opacity= 0, duration=0.1,t=self.animation)
        self.anim2_stop = Animation(_circle_size2= [0,0] ,opacity= 0, duration=0.1,t=self.animation)
        self.anim3_stop = Animation(_circle_size3= [0,0] ,opacity= 0, duration=0.1,t=self.animation)
        self.anim1_stop.start(self)
        self.anim2_stop.start(self)
        self.anim3_stop.start(self)

    def on_active(self, *args):
        size= self.size[1]
        if self.active:
            self._start_animate(size)
        else:
            self._stop_animate()      


from kivy.lang.builder import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.properties import NumericProperty, StringProperty, ListProperty, BooleanProperty, OptionProperty
from kivymd.theming import ThemableBehavior
from kivy.metrics import dp 
from kivy.uix.behaviors import ButtonBehavior
from kivymd.uix.behaviors import CircularRippleBehavior
from kivy.animation import Animation
from kivy.clock import Clock
from akivymd.uix.badgelayout import AKBadgeLayout
from kivy.core.window import Window
Builder.load_string(
    """
<Button_Item>
    size_hint: None, None 
    size: root.button_height , root.button_height
    pos_hint: {'center_x': .5, 'center_y': .5}


    canvas.before:
        Color: 
            rgba: root.button_bg_color if root.button_bg_color else app.theme_cls.primary_light
            a: root._bg_opacity
        RoundedRectangle:
            size: self.size 
            pos: self.pos 
            radius: [self.height/2,]

    AKBadgeLayout:
        badgeitem_size: root.badgeitem_size
        bg_color: root.badge_bg_color
        badgeitem_padding: root.badgeitem_padding
        badgeitem_color: root.badgeitem_color
        position: root.badge_position
        text: root.badge_text
        bold: root.badge_bold
        offset: root.badge_offset
        badge_disabled: root.badge_disabled

        MDIcon:
            id: _icon
            icon: root.icon
            halign: 'center'
            size_hint: None, None 
            size: root.height, root.height
            font_size: root.icon_size
            theme_text_color: 'Custom'
            text_color: root.icon_color if root.icon_color else app.theme_cls.primary_dark

    FloatLayout:
        id: _float
        size_hint_x: None 
        width: root.width- _icon.width
        
        Label:  
            id: _label
            text: root.text
            opacity: 0
            halign: 'center'
            color: root.text_color if root.text_color else app.theme_cls.primary_dark
            font_size: root.text_size
            size_hint: None, None 
            size: self.texture_size
            pos_hint: {'center_y': .5}
            x: _float.x + (_float.width- _label.width)/2  - (root.height- _icon.font_size)/2

<AKBottomNavigation2>:
    size_hint: None, None 
    height: root.bottomnavigation_height
    canvas.before:
        Color:
            rgba: root.bg_color if root.bg_color else app.theme_cls.bg_darkest
        RoundedRectangle:
            pos: self.pos 
            size: self.size 
            radius: [root.radius, root.radius, 0,0]
    
    BoxLayout:
        id: _button_box
    """
)

class Button_Item(ThemableBehavior,ButtonBehavior, BoxLayout):
    transition= StringProperty('out_quad')
    duration= NumericProperty(0.3)
    button_bg_color= ListProperty(None)
    button_width= NumericProperty(dp(120))
    text_size= NumericProperty(dp(13))
    text_color= ListProperty(None)
    button_height= NumericProperty(dp(40))
    icon_size= NumericProperty(dp(20))
    icon_color= ListProperty(None)
    text= StringProperty()
    icon= StringProperty()
    mode= OptionProperty('color_on_normal', options=['color_on_normal', 'color_on_active'])
    #================
    badgeitem_size= NumericProperty(dp(20))  
    badge_bg_color= ListProperty()
    badgeitem_padding= NumericProperty(dp(3))
    badgeitem_color= ListProperty()
    badge_position= StringProperty('right')
    badge_text= StringProperty('')
    badge_bold= BooleanProperty(False)
    badge_offset= NumericProperty(0.4)
    badge_disabled= BooleanProperty(False)        
    
    _bg_opacity= NumericProperty(1)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def on_release(self):
        for button in self.parent.children:
            if button==self:
                continue
            button._button_shrink()

        self._button_expand()
        return super().on_release() 

    def _button_expand(self):
        label_anim= Animation(opacity=1, transition= self.transition, duration= self.duration)
        label_anim.start(self.ids._label)

        anim= Animation(width=self.button_width, _bg_opacity=1, t=self.transition,  duration= self.duration)  
        anim.start(self)

    def _button_shrink(self):
        if self.mode=='color_on_active':
            opacity= 0
        else: 
            opacity= 1

        label_anim=  Animation(opacity=0, transition= self.transition, duration= self.duration)
        label_anim.start(self.ids._label)

        but_anim= Animation(width=self.height, _bg_opacity=opacity, t=self.transition, duration= self.duration)
        but_anim.start(self)

class AKBottomNavigation2(ThemableBehavior, BoxLayout):
    bottomnavigation_height= NumericProperty('65dp')
    radius= NumericProperty('20dp')
    bg_color= ListProperty(None)
    elelevation = NumericProperty(None)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Window.bind(on_resize=self._update )
        Clock.schedule_once(lambda x: self.set_current(None))
        Clock.schedule_once(lambda x: self._update() )

    def _update(self, *args):
        self.width= Window.width
        buttons= self.ids._button_box.children
        button_sizes= ((len(buttons)-1)* buttons[0].button_height) + buttons[0].button_width
        space= self.width - button_sizes
        spacing= space/(len(buttons)+1)
        self.ids._button_box.spacing= spacing
        self.ids._button_box.padding= [spacing,0, spacing, 0]

    def set_current(self, index):
        if not index:
            index= -1
        button= self.ids._button_box.children[index]
        button.dispatch('on_release')

    def add_widget(self, widget, index=0, canvas=None):
        if issubclass(widget.__class__, Button_Item):
            return self.ids._button_box.add_widget(widget)
        else:
            return super().add_widget(widget, index=index, canvas=canvas)



from kivymd.app import MDApp
from kivy.lang.builder import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.animation import Animation
from kivy.properties import NumericProperty , ObjectProperty , ListProperty, StringProperty
from kivy.clock import Clock
from kivymd.uix.button import MDIconButton
from kivy.metrics import dp
from kivy.core.window import Window 
from kivymd.uix.label import MDLabel
from kivymd.theming import ThemableBehavior

__all__ = ('AKBottomNavigation')

Builder.load_string( 
"""
<_AKLabel>
    size_hint:None ,None 
    size: dp(48) , dp(48)
    font_style: 'Caption' 
    halign: 'center'
    valign: 'center'
    theme_text_color: 'Custom'
    text_color: root.text_color if root.text_color else root.theme_cls.primary_light

<_AKButton>
    theme_text_color: 'Custom'
    text_color: root.icon_color if root.icon_color else root.theme_cls.primary_color

<AKBottomNavigation>:
    orientation: 'vertical'
    size_hint_y: None 
    height: self.minimum_height
    BoxLayout:
        size_hint_y: None
        height: dp(14)

        canvas.before:
            Color:
                rgba: root.bar_color if root.bar_color else root.theme_cls.primary_color
            Rectangle:
                pos: self.pos
                size: self.size

    BoxLayout:
        size_hint_y: None
        height: dp(56)

        canvas.before:
            Color:
                rgba: root.bg_color if root.bg_color else root.theme_cls.bg_dark
            Rectangle:
                pos: self.pos
                size: self.size

        Widget:
            id: _bubble
            bubble_x: 0
            size_hint: None , None 
            size: root.width , dp(70)            
            canvas.before:
                Color:
                    rgba: root.bar_color if root.bar_color else root.theme_cls.primary_color
                Rectangle:
                    pos: self.bubble_x , dp(28)
                    size: dp(112) , dp(28)
                Ellipse:
                    pos: self.bubble_x+dp(28) , 0
                    size: dp(56) , dp(56)
                Color:
                    rgba: root.bg_color if root.bg_color else root.theme_cls.bg_dark
                Ellipse:
                    pos: self.bubble_x - dp(28) , 0
                    size: dp(56) , dp(56)
                Ellipse:
                    pos: self.bubble_x + dp(84) , 0
                    size: dp(56) , dp(56)

            FloatLayout:
                id: _text_bar
                size_hint: None , None 
                size: root.width , dp(56)

            FloatLayout:
                id: _buttons_bar
                size_hint: None , None 
                size: root.width , dp(70)
            

"""
)

class _AKLabel(MDLabel):
    text_color= ListProperty()
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class _AKButton(MDIconButton):
    icon_color= ListProperty()
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda x: self._update())
        
    def on_release(self):
        self.root.set_current( self.parent.children.index(self) )

    def _update(self):
        self.root= self.parent.parent.parent.parent 

    
class AKBottomNavigation(ThemableBehavior,BoxLayout):
    items = ListProperty()
    bar_color= ListProperty()
    icon_color= ListProperty()
    text_color= ListProperty()
    bg_color= ListProperty()
    transition= StringProperty('out_sine')
    duration= NumericProperty(0.2)

    _selected = -1
    def __init__(self, **kwargs):
        super().__init__(**kwargs)  
        Window.bind(on_resize=self._on_resize)
    
    def _clear_bar(self):
        self.ids._buttons_bar.clear_widgets()
        self.ids._text_bar.clear_widgets()
        
    def _update_items(self , items):
        count = len(self.items)
        section_x = 1/(count+1)
        but_pos = section_x
        for x in range(0,count):
            button = _AKButton(
                icon=self.items[x]['icon'],
                pos_hint= {'center_x': but_pos},
                icon_color= self.icon_color
                )
            button.bind(on_release = self.items[x]['on_release'])

            label = _AKLabel(
                text=self.items[x]['text'] , 
                pos_hint= {'center_x': but_pos},
                opacity = 0,
                text_color= self.text_color
            )

            self.ids._text_bar.add_widget(label)
            self.ids._buttons_bar.add_widget(button)
            but_pos += section_x
        self.ids._bubble.bubble_x = Window.size[0]*self.ids._buttons_bar.children[self._selected].pos_hint['center_x']-dp(56)
        self.ids._buttons_bar.children[self._selected].opacity = 0      
        self.ids._text_bar.children[self._selected].opacity = 1

    def _on_resize(self , instance , width , height):
        self.ids._bubble.bubble_x = width*self.ids._buttons_bar.children[self._selected].pos_hint['center_x']-dp(56)
    
    def on_items(self,*args):
        self._clear_bar()
        return self._update_items(self.items)

    def set_current(self, index):
        current_item_button= self.ids._buttons_bar.children[index]
        current_item_text = self.ids._text_bar.children[index]
        AKBottomNavigation._selected = index

        for x in self.ids._buttons_bar.children: # button 
            x.opacity = 1

        for x in self.ids._text_bar.children: # text 
            x.opacity = 0

        bubble_pos = current_item_button.x - dp(31)
        anim_bubble = Animation(bubble_x=bubble_pos , t=self.transition , duration=self.duration)
        anim_text_opacity = Animation(opacity=1 , t=self.transition , duration=self.duration)
        anim_icon_opacity = Animation(opacity=0 , t=self.transition , duration=self.duration)

        anim_icon_opacity.start(current_item_button)
        anim_text_opacity.start(current_item_text)
        anim_bubble.start(self.ids._bubble)